Activity {
  include moving;
  successor welding;
  successor idle;
}

FactorModel {
  // accidents, incidents
  RC desc "(R)obot arm harshly (C)ollides with operator"
    mishap
    requiresNOf (1|HRW,HS,HC) 
    requires (RT)
    excludes (RT)
    prevents (RT)
    preventsMit (RT)
    detectedBy (SHARE.RCdet)
    sev=10 // severity of consequences if RC occurs
    prob=0.1; // probability of occurrence given all constraints are fulfilled
  RT desc "(R)obot arm (T)ouches the operator"
    requiresNOf (1|HRW,HS,HC)
    mitPreventsMit (RC)
    sev=5;

  // latent causes
  HRW desc "(H)uman arm and (R)obot on shared (W)orkbench"
    requires (HW)
    prevents (HC)
    mitPreventsMit (HS,HC)
    guard "hACT_WORKING & rloc=sharedTbl & hloc=sharedTbl"
    detectedBy (SHARE.HRWdet)
    mitigatedBy (PREVENT.HRWmit)
    resumedBy (.HRWres)
    sev=3;
  HW desc "(H)uman (arm) on shared (W)orkbench"
    guard "hloc=sharedTbl"
    detectedBy (SHARE.HWdet) 
    mitigatedBy (LIMIT.HWmit)
    resumedBy (.HWres)
    sev=1;
  HC // extends (moving::*).HC by
    prevents (HRW)
    mitPreventsMit (HRW,HW,RC,RT);
}

Application cobot {
  // detectors
  mode RCdet desc "touch sensor"
    guard "hACT_WORKING"; 
  mode RTdet
    prob=0.5;

  // mitigation and resumption actions
  mode HRWdet 
    guard "rloc=sharedTbl & lgtBar=true"
    embodiedBy cellObSys;
  mode HRWmit desc "safety-rated monitored stop in current activity"
    update "(notif'=leaveWrkb)" target (safmod=srmst)
    embodiedBy robotArm
    disruption=5 nuisance=1 effort=0.5;
  mode HRWres
    guard "notif=leaveWrkb & hloc!=sharedTbl" 
    update "(notif'=ok)" target (safmod=normal);

  mode HWdet 
    guard "lgtBar=true"
    embodiedBy lgtBar;
  mode HWmit desc "speed/separation monitoring" type LIMIT
    target (safmod=ssmon)
    disruption=1 effort=2;
  mode HWres desc "speed/separation monitoring"
    target (safmod=normal); // target if no other critical event is active
}
