Activity {
  include base;
}

FactorModel {
  // latent causes of accidents (critical events, risk/causal factors, hazards)
  HS desc "(H)uman in (S)afeguarded area while robot working or welding"
    /* direct */
    guard "hACT_WORKING & (hloc=inCell | hloc=atWeldSpot)"
    detectedBy (SHARE.HSdet)
    mitigatedBy (.ssmon,.srmst,.stopped)
    resumedBy (.HSres)
    mis="h_approachWeldSpot|h_exitPlant"
    prob=0.025
    sev=2;
  // latent causes
  HC desc "(H)uman (C)lose to active welder and robot working"
    requires (HS)
    mitPreventsMit (HS)
    guard "hACT_WELDING & hloc=atWeldSpot" // plant + environment state without sensor
    detectedBy (SHARE.HCdet)
    mitigatedBy (SHUTDOWN.HCStOffVis,SHUTDOWN.HCSrmstIdleVis,SHUTDOWN.HCStOffAud)
    resumedBy (NOTIFY.HCres,NOTIFY.HCres2)
    mis="h_exitPlant"
    prob=0.05 // probability of mishap if unrecognised OR active and not mitigated
    sev=5;
}

Application cobot {
  // detectors, mitigation and resumption commands ...
  // ... for HS
  mode HSdet desc "range detector"
    guard "hACT_WORKING & (rngDet=near | rngDet=close)";
  mode HSres
    guard "!hST_HOinSGA"
    target (safmod=normal);

  // ... for HC
  mode HCdet desc "light barrier"
    guard "hACT_WELDING & rngDet=close" 
    embodiedBy rngDet;
  mode HCStOffVis
    desc "emergency stop of cell / robot+welder turned off / visual notif."
    // a. synchronise with all actors on transition to act
    event stop 
    // c. apply safety function
    update "(notif'=leaveArea)"
    // d. set activity and safety mode as the mitigation target
    target (act=off, safmod=stopped)
    // e. specify mitigation characteristics [0..worst, 10..best]
    disruption=2 nuisance=5 effort=6;
  mode HCSrmstIdleVis desc "safety-rated monitored stop of robot / robot+welder go idle / visual notif."
    event halt
    update "(notif'=leaveArea)" target (act=idle, safmod=srmst)
    disruption=5 nuisance=4 effort=8; // repeatable=true
  mode HCStOffAud desc "emergency stop of cell / robot+welder turned off / audio-vis. notif."
    event stop2
    update "(notif'=leaveArea)" target (act=off, safmod=stopped)
    disruption=3 nuisance=6 effort=5;
  mode HCres desc "exchange workpiece and start over"
    // a. synchronise with all actors on transition to exchWrkp
    event resume
    // b. check whether safety function has been applied and hazard has been removed
    guard "!hST_HOinSGA" // notif=leaveArea & 
    // c. deactivate safety function
    update "(notif'=ok)"
    // d. set activity and safety mode as the resumption target
    target (act=exchWrkp, safmod=normal)
    // e. specify resumption characteristics
    disruption=2 nuisance=5 effort=1;
  mode HCres2 desc "continue welding if workpiece still unfinished"
    event resume2 guard "!hST_HOinSGA" 
    update "(notif'=ok)" target (act=welding, safmod=normal)
    disruption=7 nuisance=7 effort=7;  
}


Weights rewards {
                         guard             risk_HC;
  // robotArm
 r_moveToTable:          ""                "5";
 r_grabLeftWorkpiece: 	 ""                "0";
 r_placeWorkpieceRight:  ""                "0";
 r_moveToWelder: 	       ""                "5";
  // weldingMachine
 rw_weldStep: 		       ""                "10"; 
 rw_leaveWelder: 	       ""                "5";
  // humanOp
 h_start: 		           ""                "0";
 h_placeWorkpieceLeft: 	 ""                "0";
 h_withDrawArm: 	       ""                "0";
 h_grabRightWorkpiece:   ""                "0";
 h_enterCell:		         ""                "0";
 h_approachWeldSpot: 	   ""                "7"; 
 h_exitPlant: 		       "notif=leaveArea" "0";
}
