Activity {
  include moving;
  successor welding;
  successor idle;
}

FactorModel {
  // accidents, incidents
  /*
  RC desc "(R)obot arm harshly (C)ollides with operator"
    mishap
    requiresNOf (1|HRW,HS,HC) 
    requires (RT)
    excludes (RT)
    prevents (RT)
    preventsMit (RT)
    // guard "hACT_WORKING & (hST_HOinSGA | hloc=sharedTbl)" 
    detectedBy (SHARE.RCdet)
    sev=10 // [10..25]    // severity of consequences if RC occurs
    prob=0.1;  // [0.1..0.3]; // probability of occurrence given all constraints are fulfilled
  */
  RT desc "(R)obot arm (T)ouches the operator"
    mishap
    requiresNOf (1|HRW,HS,HC)
    /* mitPreventsMit (RC) */
    guard "hACT_WORKING"
    detectedBy (.RTdet)
    sev=5;

  // latent causes
  HRW desc "(H)uman arm and (R)obot on shared (W)orkbench"
    requires (HW)
    prevents (HC)
    mitPreventsMit (HS,HC)
    guard "hACT_WORKING & rloc=sharedTbl & hloc=sharedTbl"
    detectedBy (SHARE.HRWdet)
    mitigatedBy (PREVENT.HRWmit)
    resumedBy (.HRWres)
    mis=h_placeWorkpieceLeft|h_grabRightWorkpiece
    prob=0.01
    sev=3;
  HW desc "(H)uman (arm) on shared (W)orkbench"
    guard "hACT_WORKING & hloc=sharedTbl"
    detectedBy (SHARE.HWdet) // activatedBy (SHARE.HWdet)
    mitigatedBy (LIMIT.HWmit)
    resumedBy (.HWres)
    sev=1;
  HC // extends (moving::*).HC by
    prevents (HRW)
    mitPreventsMit (HW,HRW,RT); // RC
}

Application cobot {
  mode RCdet desc "touch/vibration sensor"
    guard "hACT_WORKING"; // TODO: include touch
  
  mode RTdet desc "touch and image sensors"
    guard "hACT_WORKING"
    prob=0.5;

  // detectors, mitigations, and resumptions
  // ... for HRW:
  mode HRWdet 
    guard "hACT_WORKING & rloc=sharedTbl & lgtBar=true"
    embodiedBy cellObSys;
  mode HRWmit desc "safety-rated monitored stop in current activity"
    update "(notif_leaveWrkb'=true)" // application of SF
    target (safmod=srmst) // mitigation to SM
    embodiedBy robotArm
    disruption=5 nuisance=1 effort=0.5;
  mode HRWres
    guard "hloc!=sharedTbl" // checking for SF deployment and HRW removal
    update "(notif_leaveWrkb'=false)" // inversion of (switching off) SF (result checked before SM resumption)
    target (safmod=normal); // resumption to productive SM

  // ... for HW:
  mode HWdet 
    guard "hACT_WORKING & lgtBar=true"
    embodiedBy lgtBar;
  mode HWmit desc "speed/separation monitoring" type LIMIT
    target (safmod=ssmon)
    disruption=1 effort=2;
  mode HWres 
    guard "hloc!=sharedTbl"
    target (safmod=normal); // target if no other CE is active
}
