Activity {
  include moving;
  successor welding;  
  successor off;      
  successor idle;     
}

FactorModel {
  // accidents, incidents
  RC desc "(R)obot arm harshly (C)ollides with operator"
    mishap
    requiresNOf (1|HRW,HS,HC) 
    requires (RT)
    excludes (RT)
    prevents (RT)
    preventsMit (RT)
    guard "hACT_WORKING & (hST_HOinSGA | hloc=sharedTbl)"
    detectedBy (SHARE.RCdet)
    sev=10 // severity of consequences if RC occurs
    prob=0.01; // probability of occurrence given all constraints are fulfilled
  RT desc "(R)obot arm (T)ouches the operator"
    requiresNOf (1|HRW,HS,HC)
    mitPreventsMit (RC)
    guard "hACT_WORKING & (hST_HOinSGA | hloc=sharedTbl)"
    sev=1;

  // latent causes
  HRW desc "(H)uman arm and (R)obot on shared (W)orkbench"
    requires (HW)
    prevents (HC)
    mitPreventsMit (HS,HC)
    guard "hACT_WORKING & rloc=sharedTbl & hloc=sharedTbl"
    detectedBy (SHARE.HRWdet)
    mitigatedBy (PREVENT.HRWmit)
    resumedBy (.HRWres)
    mis=h_placeWorkpieceLeft|h_grabRightWorkpiece
    prob=.01
    sev=3;
  HW desc "(H)uman (arm) on shared (W)orkbench"
    guard "hACT_WORKING & hloc=sharedTbl"
    detectedBy (SHARE.HWdet) 
    mitigatedBy (LIMIT.HWmit)
    resumedBy (.HWres)
    sev=1;
  HC // extends (moving::*).HC by
    prevents (HRW)
    mitPreventsMit (HW,HRW,RT,RC);
  HS excludes (HW); // no more important but feasible
}

Application cobot {
  mode RCdet desc "touch/vibration sensor"
    guard "hACT_WORKING";
  
  mode RTdet desc "touch and image sensors"
    guard "hACT_WORKING";
  mode RTmis
    target (safmod=stopped);
  mode RTres 
    target (safmod=normal);

  // detectors, mitigations, and resumptions
  // ... for HRW:
  mode HRWdet 
    guard "hACT_WORKING & rloc=sharedTbl & lgtBar=true"
    embodiedBy cellObSys;
  mode HRWmit desc "safety-rated monitored stop in current activity"
    update "(notif_leaveWrkb'=true)" // application of SF
    target (safmod=srmst) // mitigation to safety mode
    embodiedBy robotArm
    disruption=5 nuisance=1 effort=0.5;
  mode HRWres
    guard "hloc!=sharedTbl" // checking for safety function deployment and hazard removal
    update "(notif_leaveWrkb'=false)" // switching off safety function
    target (safmod=normal); // resumption to less restrictive safety mode

  // ... for HW:
  mode HWdet 
    guard "hACT_WORKING & lgtBar=true"
    embodiedBy lgtBar;
  mode HWmit desc "speed/separation monitoring" type LIMIT
    target (safmod=ssmon)
    disruption=1 effort=2;
  mode HWres 
    guard "hloc!=sharedTbl"
    target (safmod=normal); // target unless unacceptable in reached risk state 
}
