settings {
  extFile = ../01-process/workcell-tpl.pm;
  title = "Demonstrator Implementation of Safety Controller -- Draft in C-Sharp";
  note = "Created for the CSI: Cobot project";
  author = "Mario Gleirscher";
}

activity {}

application cobot {
  // helper
  hST_HOinSGA = "hloc=inCell | hloc=atWeldSpot";
  hACT_WELDING = "ract=welding & wact=welding";
  hST_HOatShTbl = "hloc=sharedTbl";
  hSM_PERM = "safmod=normal | safmod=hguid | safmod=ssmon | safmod=pflim";
  hFINAL_CUSTOM = "wpfin=1 & wps=empty & reffocc=0 & mntDone=1";

  mode rngDet;

  // interleaved logical domains
  op desc "operator" type DOMAIN;
  wc desc "work cell" type DOMAIN;

  // behaviour embodying loop components
  safetyCtr type CONTROLLER
    monVars="ract:act|wact:act|safmod|rloc:loc|lgtBar|rngDet|notif|notif_leaveWrkb"
    ctrVars="ract:act|wact:act|safmod|notif|notif_leaveWrkb";

  humanOp type HUMAN_OPERATOR partOf op;

  // physical components and interface description (sensors, actuators)
  cellObSys desc "cell observation system" role OPTICAL;
  rngDet desc "range detector" role SENSOR partOf cellObSys
    varname = "rngDet"
    vartype = "" // by default enum with same name
    channel = "distanceStatus" message = "DistanceMeasurement"
    topic   = "/safetyController/OnrngDetChange"
    code    = "rngDet.cs";
  lgtBar desc "light barrier" type SENSOR partOf cellObSys
    varname = "lgtBar" vartype = ""
    channel = "lightTriggerStatusChannel"
    message = "LightGateTrigger"
    topic   = "/safetyController/OnlgtBarChange"
    code    = "lgtBar.cs";

  weldingMachine type AGENT partOf wc
    validActs="welding|idle|off";
  wactMon type SENSOR partOf weldingMachine
    varname = "wact" vartype = "ACT"
    channel = "welderStatusChannel" message = "ACTStatus"
    topic   = "/safetyController/OnwactChange";
  wactCtr type ACTUATOR partOf weldingMachine
    varname = "wact" vartype = "ACT"
    channel = "welderModeRequestChannel" message = "WelderModeRequest"
    topic   = "/safetyController/welder/modeRequest"
    helper  = "WelderModeRequest";
  
  robotArm type AGENT partOf wc
    validActs="exchWrkp|welding|off";
  ractMon type SENSOR partOf robotArm 
    varname = "ract" vartype = "ACT"
    channel = "robotStatusChannel" message = "ACTStatus"
    topic   = "/safetyController/OnractChange";
  rlocMon type SENSOR partOf robotArm
    varname = "rloc" vartype = "LOC"
    channel = "robotLocationChannel" message = "TriggerRegionEvent"
    topic   = "/safetyController/OnrlocChange"
    code    = "rloc.cs";
  ractCtr type ACTUATOR partOf robotArm 
    varname = "ract" vartype = "ACT"
    channel = "robotModeRequestChannel" message = "RobotModeRequest"
    topic   = "/safetyController/robot/modeRequest"
    helper  = "RobotModeRequest";
  srmst_dev role ACTUATOR partOf robotArm;
  
  safmodMon type SENSOR partOf safetyCtr
    varname = "safmod" vartype = ""
    channel = "safetyModeStatusChannel" message = "SafetyModeRequest"
    topic   = "/safetyController/OnsafmodChange";
  safmodCtr type ACTUATOR partOf safetyCtr
    varname = "safmod" vartype = ""
    channel = "safetyModeRequestChannel" message = "SafetyModeRequest"
    topic   = "/safetyController/safety/modeRequest"
    helper  = "SafetyModeRequest";
  
  notif_leaveWrkbMon type SENSOR partOf safetyCtr
    desc "workbench operator notifications"
    varname = "notif_leaveWrkb" vartype = ""
    channel = "operatorWorkbenchFeedbackChannel" message = "OperatorWorkbenchInteraction"
    topic   = "/safetyController/Onnotif_leaveWrkbChange";
  notif_leaveWrkbCtr type ACTUATOR partOf safetyCtr
    desc "workbench operator notifications"
    varname = "notif_leaveWrkb" vartype = ""
    channel = "operatorWorkbenchInteractionChannel" message = "OperatorWorkbenchInteraction"
    topic   = "/safetyController/operatorWrkbFeedback"
    helper  = "OperatorWorkbenchInteraction";
  notifMon type SENSOR partOf safetyCtr
    desc "workcell operator notifications"
    varname = "notif" vartype = ""
    channel = "operatorFeedbackChannel" message = "OperatorInteraction"
    topic   = "/safetyController/OnnotifChange";
  notifCtr type ACTUATOR partOf safetyCtr
    desc "workcell operator notifications"
    varname = "notif" vartype = ""
    channel = "operatorInteractionChannel" message = "OperatorInteraction"
    topic   = "/safetyController/operatorFeedback"
    helper  = "OperatorInteraction";
}

// risk gradient matrices for activities and safety modes (skew-diagonal, relative distances)
distances act {
 off:       0;
 idle:      1 0; 
 exchWrkp:  3 2 0;
 welding:   5 4 2 0;
}
         
distances safmod {
 normal:   0;
 hguid:   -2  0;
 ssmon:   -1  1    0;
 pflim:   -2  0   -1  0;
 srmst:   -3 -1   -2 -1  0;
 stopped: -4 -2   -3 -2 -1  0;
}

weights rewards {
                         guard_prod        prod;
  // agent: robotArm
 r_moveToTable:          ""                "h";
 r_grabLeftWorkpiece: 	 ""                "m";
 r_placeWorkpieceRight:  ""                "h";
 r_moveToWelder: 	       ""                "h";
  // agent: weldingMachine
 rw_weldStep: 		       ""                "h"; 
 rw_leaveWelder: 	       ""                "h";
  // agent: human
 h_idle:                 ""                "0";
 h_start: 		           ""                "l";
 h_placeWorkpieceLeft: 	 ""                "h";
 h_withDrawArm: 	       ""                "0";
 h_grabRightWorkpiece:   ""                "h";
 h_enterCell:		         ""                "0";
 h_approachWeldSpot: 	   "mntDone=0"       "h";  // valuable maintenance action
 h_exitPlant: 		       ""                "0";  
}

// probability of mishap from factor X if factor X is active in safety mode S
weights mitFact {
          HC       HRW       HS        prod;
 normal:  "1"      "1"       "1"       "1";
 hguid:   "0.4"    "0.2"     "0.2"     "0.4";
 ssmon:   "0.2"    "0.3"     "0.4"     "0.8";
 pflim:   "0.6"    "0.3"     "0.3"     "0.7";
 srmst:   "0.1"    "0.05"    "0.1"     "0.1";
 stopped: "0"      "0"       "0"       "0";
}

enum LOC {atTable = 0, sharedTbl = 1, inCell = 2, atWeldSpot = 3}
enum RNGDET {far = 0, near = 1, close = 2}
enum NOTIF {ok = 0, leaveArea = 1}
