#!/bin/bash
##########################################################################
# YAP Tools: Script for pretty printing of PRISM's DTMC output
# CC BY-NC-SA 4.0 by Mario Gleirscher
# See http://creativecommons.org/licenses/by-nc-sa/4.0/
##########################################################################

# Usage: ${0} FILE
# Requires: ${FILE}.pst,sta,lab
# Requires: bash echo rm sed
# Provides: labelled nodes in DOT format on the standard output

FILE=${1}

INIT=0
DEADLOCK=1

# parse labels to identify state indices
for l in ANY ANYREC SAFE MISHAP FINAL; do
    declare -g $l=$(sed -n -E -e '1,1 s/^.* ([0-9]+)="'${l}'".*$/\1/p' <${FILE}.lab)
    # echo ${l}=${!l}
done

rm -f ${FILE}.tmp

# identify state pairs
for i in `sed -n -e '2,$p' <${FILE}.pst`; do
    PID=${i%:*}
    TMP=${i#*,}
    DAT="${TMP%)}"
    # search for pairs only if we deal with a product automaton
    if [[ -n `sed -E -n -e '/\(_da,/p' <${FILE}.pst` ]]; then
	ID=`sed -E -n -e '/'${DAT}'/ s/^([0-9]+):.*$/\1/p' <${FILE}.sta`
    else
	ID=${PID}
    fi
    # can only yield a single line!
    echo "${ID}=${PID}" >>${FILE}.tmp;
done

# look up the labels
IFS=$'\n'       
for s in $(cat ${FILE}.tmp); do
    ID=${s%=*}
    PID=${s#*=}
    # DEADLOCK
    # sed -E -n -e '/^'$ID':/ s/^[0-9]+:.*\s'$DEADLOCK'(\s.*)?$/'$PID' [label="'$PID'",color=orange];/p' <${FILE}.lab 
    # mark UNSAFE states
    sed -E -n -e '/^'$ID':/ s/^[0-9]+:.*\s'$ANY'(\s.*)?$/'$PID' [label="'$PID'",color=gray];/p' <${FILE}.lab 
    # mark recognised UNSAFE states
    sed -E -n -e '/^'$ID':/ s/^[0-9]+:.*\s'$ANYREC'(\s.*)?$/'$PID' [label="'$PID'",color=orange];/p' <${FILE}.lab 
    # SAFE
    sed -E -n -e '/^'$ID':/ s/^[0-9]+:.*\s'$SAFE'(\s.*)?$/'$PID' [label="'$PID'",color=green];/p' <${FILE}.lab 
    # MISHAP
    sed -E -n -e '/^'$ID':/ s/^[0-9]+:.*\s'$MISHAP'(\s.*)?$/'$PID' [label="'$PID'",color=red];/p' <${FILE}.lab 
    # INITIAL
    sed -E -n -e '/^'$ID':/ s/^[0-9]+:.*\s'$INIT'(\s.*)?$/'$PID' [label="'$PID'",color=CornflowerBlue];/p' <${FILE}.lab 
    # FINAL
    sed -E -n -e '/^'$ID':/ s/^[0-9]+:.*\s'$FINAL'(\s.*)?$/'$PID' [label="'$PID'",color=YellowGreen];/p' <${FILE}.lab 
done
